var _cheerio = require('cheerio');

var _cheerio2 = _interopRequireDefault(_cheerio);

var _fs = require('fs');

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _lodash = require('lodash');

var _CONFIG = require('./CONFIG');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var getEntryHtml = function getEntryHtml(entry, rootPath) {
  var filePath = _path2['default'].join((0, _CONFIG.ssrDir)(rootPath), entry);
  if ((0, _fs.existsSync)(filePath) && (0, _fs.lstatSync)(filePath).isDirectory()) {
    filePath = String(filePath) + '/index.html';
  } else {
    filePath = String(filePath) + '.html';
  }

  return (0, _fs.readFileSync)(filePath, 'utf8');
};

var updatePreloadedState = function updatePreloadedState($, newState) {
  $('script').get().forEach(function (script) {
    // find script with `data-coren` attr
    if ((0, _lodash.has)(script.attribs, 'data-coren')) {
      var preloadStateTxt = script.children[0].data;
      var preloadState = JSON.parse(preloadStateTxt.replace('window.__PRELOADED_STATE__ =', ''));
      var mergeState = Object.assign({}, preloadState, newState);
      script.children[0].data = 'window.__PRELOADED_STATE__ = ' + JSON.stringify(mergeState);
    }
  });
  return $;
};

module.exports = function (rootPath) {
  return function () {
    function corenMiddleware(req, res, next) {
      var setHead = null;
      var preloadedState = null;

      res.sendCoren = function (entry) {
        // multi routes render
        // from index/users/1 => users/index/1
        if (entry.indexOf('/') >= 0) {
          var entries = entry.split('/').filter(function (val) {
            return val;
          });

          // multi routes render
          if (entries.length > 1) {
            var firstEntry = entries[0];
            var newEntries = entries.slice(1);
            newEntries.push(firstEntry);
            entry = newEntries.join('/');
          }
        }

        var $ = _cheerio2['default'].load(getEntryHtml(entry, rootPath));
        if (preloadedState) {
          $ = updatePreloadedState($, preloadedState);
        }
        if (setHead) {
          var api = {
            append: function () {
              function append(content) {
                $('head').append(content);
              }

              return append;
            }()
          };
          setHead(api);
        }
        res.send($.html());

        // init head & preloadedState
        setHead = null;
        preloadedState = null;
      };

      res.setHead = function (cb) {
        setHead = cb;
      };

      res.setPreloadedState = function (obj) {
        preloadedState = obj;
      };
      next();
    }

    return corenMiddleware;
  }();
};